@@ -24,7 +24,7 @@ set -euo pipefail
 skip_without_bwrap
 skip_revokefs_without_fuse
 
-echo "1..16"
+echo "1..18"
 
 # Use stable rather than master as the branch so we can test that the run
 # command automatically finds the branch correctly
@@ -80,6 +80,36 @@ assert_file_has_content runtime-fpi "^runtime=runtime/org\.test\.Platform/$ARCH/
 
 ok "run a runtime"
 
+if [ -f /etc/os-release ]; then
+    run_sh org.test.Platform cat /run/host/os-release >os-release
+    (cd /etc; md5sum os-release) | md5sum -c
+
+    ARGS="--filesystem=host-etc" run_sh org.test.Platform cat /run/host/os-release >os-release
+    (cd /etc; md5sum os-release) | md5sum -c
+
+    if run_sh org.test.Platform "echo test >> /run/host/os-release"; then exit 1; fi
+    if run_sh org.test.Platform "echo test >> /run/host/os-release"; then exit 1; fi
+elif [ -f /usr/lib/os-release ]; then
+    run_sh org.test.Platform cat /run/host/os-release >os-release
+    (cd /usr/lib; md5sum os-release) | md5sum -c
+
+    ARGS="--filesystem=host-os" run_sh org.test.Platform cat /run/host/os-release >os-release
+    (cd /usr/lib; md5sum os-release) | md5sum -c
+
+    if run_sh org.test.Platform "echo test >> /run/host/os-release"; then exit 1; fi
+    if run_sh org.test.Platform "echo test >> /run/host/os-release"; then exit 1; fi
+fi
+
+ok "host os-release"
+
+run_sh org.test.Platform 'cat /run/host/container-manager' > container-manager
+echo flatpak > expected
+diff -u expected container-manager
+run_sh org.test.Platform 'echo "${container}"' > container-manager
+diff -u expected container-manager
+
+ok "host container-manager"
+
 if run org.test.Nonexistent 2> run-error-log; then
     assert_not_reached "Unexpectedly able to run non-existent runtime"
 fi
